@@ -62,12 +62,19 @@ def intersect_mesh_rays(mesh, points, vectors, normals=None, parallel=False):
             parallel with one point per CPU. (Default: False).
 
     Returns:
-        A 2D matrix of 0's and 1's indicating the results of the intersection.
-        Each sub-list of the matrix represents one of the points and has a
-        length equal to the vectors. 0 indicated a blocked ray and 1 indicates
-        a ray that was not blocked.
+        A tuple with two elements
+
+        -   intersection_matrix -- A 2D matrix of 0's and 1's indicating the results
+            of the intersection. Each sub-list of the matrix represents one of the
+            points and has a length equal to the vectors. 0 indicates a blocked
+            ray and 1 indicates a ray that was not blocked.
+
+        -   angle_matrix -- A 2D matrix of angles in radians. Each sub-list of the
+            matrix represents one of the normals and has a length equal to the
+            supplied vectors. Will be None if no normals are provided.
     """
-    int_matrix = [0] * len(points)  # matrix to be filled with results
+    intersection_matrix = [0] * len(points)  # matrix to be filled with results
+    angle_matrix = [0] * len(normals) if normals is not None else None
     cutoff_angle = math.pi / 2  # constant used in all normal checks
 
     def intersect_point(i):
@@ -78,20 +85,24 @@ def intersect_mesh_rays(mesh, points, vectors, normals=None, parallel=False):
             ray = rg.Ray3d(pt, vec)
             is_clear = 0 if rg.Intersect.Intersection.MeshRay(mesh, ray) >= 0 else 1
             int_list.append(is_clear)
-        int_matrix[i] = int_list
+        intersection_matrix[i] = int_list
 
     def intersect_point_normal_check(i):
         """Intersect all of the vectors of a given point with a normal check."""
         pt, normal_vec = points[i], normals[i]
         int_list = []
+        angle_list = []
         for vec in vectors:
-            if rg.Vector3d.VectorAngle(normal_vec, vec) <= cutoff_angle:
+            vec_angle = rg.Vector3d.VectorAngle(normal_vec, vec)
+            angle_list.append(vec_angle)
+            if vec_angle <= cutoff_angle:
                 ray = rg.Ray3d(pt, vec)
                 is_clear = 0 if rg.Intersect.Intersection.MeshRay(mesh, ray) >= 0 else 1
                 int_list.append(is_clear)
             else:  # the vector is pointing behind the surface
                 int_list.append(0)
-        int_matrix[i] = int_list
+        intersection_matrix[i] = int_list
+        angle_matrix[i] = angle_list
 
     if normals is not None:
         if parallel:
@@ -105,39 +116,49 @@ def intersect_mesh_rays(mesh, points, vectors, normals=None, parallel=False):
         else:
             for i in range(len(points)):
                 intersect_point(i)
-    return int_matrix
+    return intersection_matrix, angle_matrix
 
 
-def vector_angles(normals, vectors, parallel=False):
-    """Compute the angles between a list of normals and vectors.
+def intersect_mesh_lines(mesh, start_points, end_points, parallel=False):
+    """Intersect a group of lines (represented by end points) with a mesh.
 
-    Useful as a post-processing step from the intersect_mesh_rays method.
+    All combinations of lines that are possible between the input start_points and
+    end_points will be intersected. This method exists since most CAD plugins have
+    much more efficient mesh/line intersection functions than ladybug_geometry.
+    However, the ladybug_geometry Face3D.intersect_line_ray() method provides
+    a workable (albeit very inefficient) alternative to this if it is needed.
 
     Args:
-        normals: An array of Rhino vectors representing the direction something
-            is facing.
-        vectors: An array of Rhino vectors for which angles with the normals will
-            be computed.
+        mesh: A Rhino mesh that can block the lines.
+        start_points: An array of Rhino points that will be used to generate lines.
+        end_points: An array of Rhino points that will be used to generate lines.
         parallel: Boolean to indicate if the intersection should be run in
-            parallel with one normal per CPU. (Default: False).
+            parallel with one point per CPU. (Default: False).
 
     Returns:
-        A 2D matrix of angles in radians. Each sub-list of the matrix represents
-        one of the normals and has a length equal to the vectors.
+        A 2D matrix of 0's and 1's indicating the results of the intersection.
+        Each sub-list of the matrix represents one of the points and has a
+        length equal to the end_points. 0 indicates a blocked ray and 1 indicates
+        a ray that was not blocked.
     """
-    angle_matrix = [0] * len(normals)  # matrix to be filled with results
+    int_matrix = [0] * len(start_points)  # matrix to be filled with results
 
-    def normal_angle(i):
-        """Intersect all of the vectors of a given point with a normal check."""
-        normal_vec = normals[i]
-        angle_matrix[i] = [rg.Vector3d.VectorAngle(normal_vec, vec) for vec in vectors]
+    def intersect_start_point(i):
+        """Intersect all of the vectors of a given point without any normal check."""
+        pt = start_points[i]
+        int_list = []
+        for ept in end_points:
+            lin = rg.Line(pt, ept)
+            is_clear = 1 if rg.Intersect.Intersection.MeshLine(mesh, lin)[1] is None else 0
+            int_list.append(is_clear)
+        int_matrix[i] = int_list
 
     if parallel:
-        tasks.Parallel.ForEach(range(len(normals)), normal_angle)
+        tasks.Parallel.ForEach(range(len(start_points)), intersect_start_point)
     else:
-        for i in range(len(normals)):
-            normal_angle(i)
-    return angle_matrix
+        for i in range(len(start_points)):
+            intersect_start_point(i)
+    return int_matrix
 
 
 def intersect_solids_parallel(solids, bound_boxes):
